"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const antlr4ts_1 = require("antlr4ts");
const Interval_1 = require("antlr4ts/misc/Interval");
class Scanner {
    constructor(input) {
        this.index = 0;
        input.fill();
        this.tokens = input.getTokens();
        this.tokenStack = [];
    }
    /**
     * Advances to the next token.
     *
     * @param skipHidden If true ignore hidden tokens.
     * @return False if we hit the last token before we could advance, true otherwise.
     */
    next(skipHidden) {
        while (this.index < this.tokens.length - 1) {
            ++this.index;
            if (this.tokens[this.index].channel === antlr4ts_1.Token.DEFAULT_CHANNEL || !skipHidden) {
                return true;
            }
        }
        return false;
    }
    /**
     * Returns to the previous token.
     *
     * @param skipHidden If true ignore hidden tokens.
     * @return False if we hit the last token before we could fully go back, true otherwise.
     */
    previous(skipHidden) {
        while (this.index > 0) {
            --this.index;
            if (this.tokens[this.index].channel === 0 || !skipHidden) {
                return true;
            }
        }
        return false;
    }
    /**
     * Advances to the token that covers the given zero-based offset.
     *
     * Note: this function also considers hidden token.
     *
     * @return True if such a node exists, false otherwise (no change performed then).
     */
    advanceToPosition(offset) {
        if (this.tokens.length === 0) {
            return false;
        }
        let i = 0;
        for (; i < this.tokens.length; i++) {
            const token = this.tokens[i];
            const tokenOffset = token.tokenIndex;
            const tokenLength = token.stopIndex - token.startIndex + 1;
            if (tokenOffset <= offset && offset < tokenOffset + tokenLength) {
                this.index = i;
                break;
            }
            if (tokenOffset > offset) {
                // We reached a token after the current offset. Take the previous one as result then.
                if (i === 0) {
                    return false;
                }
                this.index = i - 1;
                break;
            }
        }
        if (i === this.tokens.length) {
            this.index = i - 1; // Nothing found, take the last token instead.
        }
        return true;
    }
    /**
     * Advances to the next token with the given lexical type.
     *
     * @param type The token type to search.
     * @return True if such a node exists, false otherwise (no change performed then).
     */
    advanceToType(type) {
        for (let i = this.index; i < this.tokens.length; ++i) {
            if (this.tokens[i].type === type) {
                this.index = i;
                return true;
            }
        }
        return false;
    }
    /**
     * Steps over a number of tokens and positions.
     * The tokens are traversed in exactly the given order without intermediate tokens. The current token must be
     * startToken. Any non-default channel token is skipped before testing for the next token in the sequence.
     *
     * @return True if all the given tokens were found and there is another token after the last token
     *         in the list, false otherwise. If the token sequence could not be found or there is no more
     *         token the internal state is undefined.
     */
    skipTokenSequence(sequence) {
        if (this.index >= this.tokens.length) {
            return false;
        }
        for (const token of sequence) {
            if (this.tokens[this.index].type != token) {
                return false;
            }
            while (++this.index < this.tokens.length && this.tokens[this.index].channel != antlr4ts_1.Token.DEFAULT_CHANNEL) {
                if (this.index === this.tokens.length) {
                    return false;
                }
            }
        }
        return true;
    }
    /**
     * Returns the type of the next token without changing the internal state.
     */
    lookAhead(skipHidden) {
        let index = this.index;
        while (index < this.tokens.length - 1) {
            ++index;
            if (this.tokens[index].channel === antlr4ts_1.Token.DEFAULT_CHANNEL || !skipHidden) {
                return this.tokens[index].type;
            }
        }
        return antlr4ts_1.Token.INVALID_TYPE;
    }
    /**
     * Look back in the stream (physical order) what was before the current token, without
     * modifying the current position.
     */
    lookBack(skipHidden) {
        let index = this.index;
        while (index > 0) {
            --index;
            if (this.tokens[index].channel === antlr4ts_1.Token.DEFAULT_CHANNEL || !skipHidden) {
                return this.tokens[index].type;
            }
        }
        return antlr4ts_1.Token.INVALID_TYPE;
    }
    seek(index) {
        if (index < this.tokens.length) {
            this.index = index;
        }
    }
    /**
     * Resets the walker to be at the original location.
     */
    reset() {
        this.index = 0;
        while (this.tokenStack.length > 0) {
            this.tokenStack.pop();
        }
    }
    /**
     * Store the current node on the stack, so we can easily come back when needed.
     */
    push() {
        this.tokenStack.push(this.index);
    }
    /**
     * Returns to the location at the top of the token stack (if any).
     */
    pop() {
        if (this.tokenStack.length === 0) {
            return false;
        }
        this.index = this.tokenStack.pop() || 0;
        this.tokenStack.pop();
        return true;
    }
    /**
     * Removes the current top of stack entry without restoring the internal state.
     * Does nothing if the stack is empty.
     */
    removeTos() {
        if (this.tokenStack.length > 0) {
            this.tokenStack.pop();
        }
    }
    /**
     * Returns true if the current token is of the given type.
     */
    is(type) {
        return this.tokens[this.index].type === type;
    }
    /**
     * Returns the textual expression of the token.
     */
    tokenText() {
        return this.tokens[this.index].text;
    }
    /**
     * Returns the type of the current token. Same as the type you can specify in advanceToType().
     */
    tokenType() {
        return this.tokens[this.index].type;
    }
    /**
     * Returns the (zero-based) index of the current token within the input.
     */
    tokenIndex() {
        return this.tokens[this.index].tokenIndex;
    }
    /**
     * Returns the (one-based) line number of the token.
     */
    tokenLine() {
        return this.tokens[this.index].line;
    }
    /**
     * Returns the offset of the current token in its source string.
     */
    tokenStart() {
        return this.tokens[this.index].startIndex;
    }
    /**
     * Returns the length of the current token in bytes.
     */
    tokenLength() {
        const token = this.tokens[this.index];
        return token.stopIndex - token.startIndex + 1;
    }
    /**
     * Returns the channel of the current token.
     */
    tokenChannel() {
        return this.tokens[this.index].channel;
    }
    /**
     * This is a special purpose function to return all the input text from the current token to the end.
     */
    tokenSubText() {
        var _a;
        const charStream = (_a = this.tokens[this.index].tokenSource) === null || _a === void 0 ? void 0 : _a.inputStream;
        const interval = new Interval_1.Interval(this.tokens[this.index].startIndex, Number.MAX_SAFE_INTEGER);
        return charStream === null || charStream === void 0 ? void 0 : charStream.getText(interval);
    }
}
exports.Scanner = Scanner;
//# sourceMappingURL=scanner.js.map